//---------------------------------------------------
//check input Matrix
//---------------------------------------------------

function [A] = check_input(features,classes)
    [features_y,features_x] = size(features);
    [classes_y,classes_x] = size(classes);
    A=1;
    if features_y <> classes_x then,
        error ('Featurematrix and Classmatrix do not fit!');
        A=0;   
    end,
    if classes_y > 1 then,
        error ('Classmatrix has wrong size! Has to be 1xN');
        A=0;      
    end,
    

endfunction;

//---------------------------------------------------
//write test temp file
//---------------------------------------------------
function [] = write_testfile(tempfile,test)
    [y,x] = size(test);
    fd=mopen(tempfile,'w');
        for i=1:y,
            line = '0 1:'+string(test(i,1));
            for j=2:x,
                line = line + ' '+string(j) + ':' + string(test(i,j));
            end,
            mputl(line,fd);
        end,
    mclose(fd);
endfunction;

//---------------------------------------------------
//file to matrix conversion
//---------------------------------------------------
function [A] = filetomatrix(infile)
    fd = mopen(infile,'r');
    B = mgetl(infile);
    A= evstr(B);
    mclose(fd);
endfunction;

//---------------------------------------------------
//write tempfile
//---------------------------------------------------

function [B] = write_infile(tempfile,features,varargin)
    B=0;
    [lhs,rhs]=argn(0);
    if rhs==2 then,
        [y,x] = size(features);
        
        fd=mopen(tempfile,'w');
        for i=1:y,
            line = string(features(i,1));
            for j=2:x,
                line = line + ' ' +string(j-1) + ':' + string(features(i,j))';
            end,
            mputl(line,fd);
        end,
        mclose(fd);
        B=1;
    end,

    if rhs==3 then,
        A =  check_input(features,varargin(1));
        if A == 1 then,
            [y,x] = size(features);
            fd=mopen(tempfile,'w');
            for i=1:y,
                line = string(varargin(1)(i));
                for j=1:x,
                    line = line +' '+ string(j) + ': ' + string(features(i,j));
                end;
                mputl(line,fd);
            end;   

            mclose(fd);
            B=1;
        end,
    end,
    
endfunction;

//---------------------------------------------------
//tail of C++ code output
//---------------------------------------------------
function [] = tail(tail_file, stop_file,old)
    //disp('old: '+string(old));
    new=old;
    last = 0;
    while (new==old|last==0),
        fd=mopen(tail_file,'r');
        [i1,i2] = size(mgetl(fd));
        now = i1;
        n = string(now-last); 
        unix_w('tail -n ' + n + ' ' + tail_file);
        last = now;
        mclose(fd);
        new=fileinfo(stop_file);
        //dt=getdate(w(6));
        //new=dt(9)+dt(8)+dt(7);
        //disp(new);
    end;
endfunction;


//--------------------------------------------------
//addcol
//--------------------------------------------------
function [A] = addcol(B,c)
    [Bm,Bn] = size(B);
    [cm,cn] = size(c);
    if Bn == cn then,
        A = [B;c];
    else,
        if Bn > cn then,
            c = [c, zeros(cm,Bn-cn)];
        end,
        if Bn < cn then,
            B = [B, zeros(Bm,cn-Bn)];
        end,
        A = [B;c];
    end,    

endfunction;

//--------------------------------------------------
//read_train_file
//-------------------------------------------------
function [A] = read_train_file(infile)
    [i1,i2]=size(fileinfo(infile));
    if i1 == 0 then,
        error('file ' + infile + ' does not exist!');
    end,
    A = [];
    fd = mopen(infile,'r');
    Temp = mgetl(fd);
    [t1,t2] = size(Temp);
    for i=1:t1,
        temp = Temp(i,1);
        temp = temp + ' ';
        l = length(temp);
        col = strindex(temp,':');
        colnum = length(col);
        if colnum <> 0 then,
        
            space = strindex(temp,' ');
            index = string(ones(1,colnum));
            values = string(ones(1,colnum));
            label = part(temp,[1:space(1)]);
            for j=1:colnum,
                index(j) = part(temp,[space(j)+1:col(j)-1]);
                values(j) = part(temp,[col(j)+1:space(j+1)-1]);
            end;
            a = zeros(1,colnum+1);
            label=eval(label);
            a(1)=label;
            index=eval(index);
            values=eval(values);
            for j=1:colnum,
                a(index(j)+1)=values(j);
            end;
            A = addcol(A,a);
        end;    
    end;    
    mclose(fd);
endfunction;

//---------------------------------------------------
//svm-train
//---------------------------------------------------

function [C] = svm_train(model,param,varargin)
    C=1;
    [lhs,rhs]=argn(0);
    if type(model) <> 10 then,
        error ('Argument 1 has wrong type: filename required');
        C=0;
    end,
    if type(varargin(1)) == 10 then,  //train from file
        infile = varargin(1);
        [i1,i2]=size(fileinfo(infile));
        if i1 == 0 then,
            error('file ' + infile + ' does not exist!');
        end,    
        
    end,
    if type(varargin(1)) == 1 then,   //train from matrix
        if rhs==3 then,
            write_infile('tempfile2.sce',varargin(1));
        end,
        if rhs==4 then,
            write_infile('tempfile2.sce',varargin(1),varargin(2));
        end,    
        infile = 'tempfile2.sce';    
    end,

    host('rm '+model);
    //fd = mopen(model,'w');
    //mclose(fd);
    //out = 'svm-train ' + string(param) + ' ' + string(infile) + ' ' + string(model);
    
    disp('SVM-TRAIN: model in file: '+model);
    old=fileinfo(model);
    //dt=getdate(w(6));
    //old=dt(9)+dt(8)+dt(7);
    host(svm_path+'/svm-train ' + string(param) + ' ' + string(infile) + ' ' + string(model)+' > '+path+'/ausgabe &');
    tail(path+'/ausgabe',model,old);
    disp('training done ...');
endfunction;

//------------------------------------------------------
//svm-predict
//------------------------------------------------------
function [A] = svm_predict(model,test,varargin)
    [lhs,rhs]=argn(0);

    if type(model) <> 10 then,
        error ('Argument 1 has wrong type: filename required');
    end,
    
    if type(test) == 10 then,  //test from file
        infile = test;
        [i1,i2]=size(fileinfo(infile));
        if i1 == 0 then,
            error('file ' + infile + ' does not exist!');
        end,
    end,

    if type(test) == 1 then, //test from matrix
        write_testfile('tempfile.sce',test);
        infile = 'tempfile.sce';
    end,    
    
    if rhs==2 then,
        disp('classifying ...');
        host(svm_path+'/svm-predict ' + string(infile) + ' ' + string(model) + '  tempout.sce > '+path+'/ausgabe');
        unix_w('cat '+path+'/ausgabe');
        A = filetomatrix('tempout.sce');
        disp('done ...');
    end,
    if rhs==3 then,
        disp('Writing output to:'+ string(varargin(1)));
        disp('classifying ...');
        host(svm_path+'/svm-predict ' + string(infile) + ' ' + string(model) + ' ' + string(varargin(1))+' > '+path+'/ausgabe');
        unix_w('cat '+path+'/ausgabe');
        A = filetomatrix(varargin(1));
        disp('done ...');
        
    end,
    if rhs>3 then error('Too many arguments !'),end;
    
endfunction;
//------------------------------------------------------
//svm-train menu
//------------------------------------------------------
function [ok] = x_svm_train()
    labels=["input file";"model output file";"svm_type";"kernel_type";"degree";"gamma";"coef0";"cost";"nu";"epsilon"];
    types=list("str",1,"str",1,"str",1,"str",1,"str",1,"str",1,"str",1,"str",1,"str",1,"str",1);
    values=["heart_scale";"svm.model";"0";"2";"3";"20";"0";"1";"0.5";"0.001"];
    [ok,infile,modelfile,s,t,d,g,r,c,n,e]=getvalue("Set svm-train parameters",labels,types,values);
    param = ' -s ' + s + ' -t ' + t + ' -d ' + d + ' -r ' + r + ' -c ' + c + ' -n ' + n + ' -e ' + e;
    if ok == %t  then,
       svm_train(modelfile,param,infile);
    end,
    
endfunction;

//------------------------------------------------------
//svm-predict menu
//------------------------------------------------------
function [ok] = x_svm_predict()
    labels=["model file";"test file";"output file"];
     types=list("str",1,"str",1,"str",1);
     values=["svm.model";"svm.test";"svm.result"];
     [ok,model,testfile,outfile]=getvalue("svm-predict",labels,types,values);
     if ok == %t  then,
            svm_predict(model,testfile,outfile);
     end,
endfunction

//-----------------------------------------------------
//menu-demo
//-----------------------------------------------------
function [C] = democall1()
    C=1;
    A=read_train_file("data1.mod");
    Train = A(:,2:3);
    Labels=A(:,1)';
    plot2d([0,1],[0,1],[-1,1]) //show graphic window
    [h,b]=size(Train);
    xrects([Train';ones(1,h)*0.02;ones(1,h)*0.02],Labels);//draw training vectors
    demo1("1","3","1","0","1000","0.5","0.001",Train,Labels);//call gui
endfunction;

function [C] = democall2()
    C=1;
    A=read_train_file("data2.mod");
    Train = A(:,2:3);
    Labels=A(:,1)';
    plot2d([0,1],[0,1],[-1,1]) //show graphic window
    [h,b]=size(Train);
    xrects([Train';ones(1,h)*0.02;ones(1,h)*0.02],Labels);//draw training vectors
    demo2("1","3","1","0","1000","0.5","0.001",Train,Labels);//call gui
endfunction;

function [C] = democall3()
    C=1;
    A=read_train_file("data3.mod");
    Train = A(:,2:3);
    Labels=A(:,1)';
    plot2d([0,1],[0,1],[-1,1]) //show graphic window
    [h,b]=size(Train);
    xrects([Train';ones(1,h)*0.02;ones(1,h)*0.02],Labels);//draw training vectors
    demo3("1","3","1","0","1000","0.5","0.001",Train,Labels);//call gui
endfunction;

function [ok] = demo1(t,d,g,r,c,n,e,Train,Labels)
    labels=["kernel_type";"degree";"gamma";"coef0";"cost";"epsilon"];
    types=list("str",1,"str",1,"str",1,"str",1,"str",1,"str",1);
    values=[t;d;g;r;c;e];
    [ok,t,d,g,r,c,e]=getvalue("Set svm-train parameters",labels,types,values);
    param = ' -t ' + t + ' -d ' + d + ' -r ' + r + ' -c ' + c + ' -n ' + n + ' -e ' + e + ' -g ' +g;
    T=[0 0];
    for y=0:0.0125:1,
        for x=0:0.0125:1,
            T=[T;[y x]];
        end;
    end;
    if ok == %t  then,
       xclear();
       svm_train("svm.model",param,"data1.mod");//[Labels' Train]);
       L=svm_predict("svm.model",T);
       [h,b]=size(T);
       xrects([T';ones(1,h)*0.02;ones(1,h)*0.02],L+10);
       [h,b]=size(Train);
       xrects([Train';ones(1,h)*0.02;ones(1,h)*0.02],Labels);
       plot2d([0,1],[0,1],[-1,1],"022") //show graphic window
       demo1(t,d,g,r,c,n,e);
    end,
    xdel();
endfunction;

function [ok] = demo2(t,d,g,r,c,n,e,Train,Labels)
    labels=["kernel_type";"degree";"gamma";"coef0";"cost";"epsilon"];
    types=list("str",1,"str",1,"str",1,"str",1,"str",1,"str",1);
    values=[t;d;g;r;c;e];
    [ok,t,d,g,r,c,e]=getvalue("Set svm-train parameters",labels,types,values);
    param = ' -t ' + t + ' -d ' + d + ' -r ' + r + ' -c ' + c + ' -n ' + n + ' -e ' + e + ' -g ' +g;
    T=[0 0];
    for y=0:0.0125:1,
        for x=0:0.0125:1,
            T=[T;[y x]];
        end;
    end;
    if ok == %t  then,
       xclear(0);
       svm_train("svm.model",param,"data2.mod");//[Labels' Train]);
       L=svm_predict("svm.model",T);
       [h,b]=size(T);
       xrects([T';ones(1,h)*0.02;ones(1,h)*0.02],L+10);
       [h,b]=size(Train);
       xrects([Train';ones(1,h)*0.02;ones(1,h)*0.02],Labels);
       plot2d([0,1],[0,1],[-1,1],"022") //show graphic window
       demo2(t,d,g,r,c,n,e);
    end,
    xdel();
endfunction;

function [ok] = demo3(t,d,g,r,c,n,e,Train,Labels)
    labels=["kernel_type";"degree";"gamma";"coef0";"cost";"epsilon"];
    types=list("str",1,"str",1,"str",1,"str",1,"str",1,"str",1);
    values=[t;d;g;r;c;e];
    [ok,t,d,g,r,c,e]=getvalue("Set svm-train parameters",labels,types,values);
    param = ' -t ' + t + ' -d ' + d + ' -r ' + r + ' -c ' + c + ' -n ' + n + ' -e ' + e + ' -g ' +g;
    T=[0 0];
    for y=0:0.0125:1,
        for x=0:0.0125:1,
            T=[T;[y x]];
        end;
    end;
    if ok == %t  then,
       xclear(0);
       svm_train("svm.model",param,"data3.mod");//[Labels' Train]);
       L=svm_predict("svm.model",T);
       [h,b]=size(T);
       xrects([T';ones(1,h)*0.02;ones(1,h)*0.02],L+10);
       [h,b]=size(Train);
       xrects([Train';ones(1,h)*0.02;ones(1,h)*0.02],Labels);
       plot2d([0,1],[0,1],[-1,1],"022") //show graphic window
       demo3(t,d,g,r,c,n,e);
    end,
    xdel();
endfunction;


//------------------------------------------------------
//menu
//------------------------------------------------------
function [SCIsvm] = svm_menu()
    global path;
    global svm_path;
    svm_path = './';
    path=pwd();
    addmenu('SCIsvm',['HELP';'set working_directory';'set libsvm path';'TRAIN';'PREDICT';'DEMO1';'DEMO2';'DEMO3';'QUIT']);
    global SCIsvm; 
    SCIsvm = ['help SCIsvm';'old_path=path;path=x_dialog([''enter current working directory:''],path);[a,b]=size(path);if a==0 then, path=old_path;end, chdir(path)';'old_svm_path=svm_path;svm_path=x_dialog([''set path of the libsvm files:''],svm_path);[a,b]=size(svm_path);if a==0 then, svm_path=old_svm_path;end,';'x_svm_train()';'x_svm_predict()';'democall1()';'democall2()';'democall3()';'delmenu(''SCIsvm'')';];
endfunction;
